﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Browser;
using System.Collections.Generic;

namespace SilverlightApplication5
{
  public static class HtmlElementExtensions
  {
    public static void Add(this HtmlElement parent, IEnumerable<HElement> elements)
    {
      foreach (HElement item in elements)
      {
        parent.AppendChild(item);
      }
    }
    public static void Add(this HtmlElement parent, params HElement[] elements)
    {
      foreach (HElement item in elements)
      {
        parent.AppendChild(item);
      }
    }
  }
  public class HAttribute
  {
    public HAttribute()
    {
    }
    public HAttribute(string name, string value)
    {
      this.Name = name;
      this.Value = value;
    }
    public string Name { get; set; }
    public string Value { get; set; }
  }
  public class HStyleAttribute : HAttribute
  {
  }
  public class HHandler
  {
    public HHandler()
    {

    }
    public HHandler(string eventName, EventHandler<HtmlEventArgs> handler)
    {
      EventName = eventName;
      Handler = handler;
    }
    public string EventName { get; set; }
    public EventHandler<HtmlEventArgs> Handler { get; set; }
  }
  public class HElement
  {
    public HElement(HtmlElement parent, string elementType, params object[] items) : this(elementType, items)
    {
      parent.AppendChild(element);
    }
    public HElement(string elementType, params object[] items)
    {
      element = HtmlPage.Document.CreateElement(elementType);
      AddItems(items);
    }
    void AddItems(object[] items)
    {
      foreach (object o in items)
      {
        if (o.GetType() == typeof(HElement))
        {
          AddSubElement((HElement)o);
        }
        else if (o.GetType() == typeof(HAttribute))
        {
          AddAttribute((HAttribute)o);
        }
        else if (o.GetType() == typeof(HStyleAttribute))
        {
          AddStyleAttribute((HStyleAttribute)o);
        }
        else if (o.GetType() == typeof(HHandler))
        {
          AddHandler((HHandler)o);
        }
        else
        {
          IEnumerable<HElement> elementList = o as IEnumerable<HElement>;

          if (elementList != null)
          {
            foreach (HElement el in elementList)
            {
              AddSubElement(el);
            }
          }
          else
          {
            IEnumerable<HAttribute> attrList = o as IEnumerable<HAttribute>;

            if (attrList != null)
            {
              foreach (HAttribute attr in attrList)
              {
                AddAttribute(attr);
              }
            }
            else
            {
              IEnumerable<HStyleAttribute> styleList = o as IEnumerable<HStyleAttribute>;

              if (styleList != null)
              {
                foreach (HStyleAttribute style in styleList)
                {
                  AddStyleAttribute(style);
                }
              }
              else
              {
                throw new InvalidOperationException("Parameter type not supported");
              }
            }
          }
        }
      }
    }

    private void AddHandler(HHandler handler)
    {
      element.AttachEvent(handler.EventName, handler.Handler);
    }

    private void AddStyleAttribute(HStyleAttribute styleAttribute)
    {
      element.SetStyleAttribute(styleAttribute.Name, styleAttribute.Value);
    }

    private void AddAttribute(HAttribute attribute)
    {
      element.SetAttribute(attribute.Name, attribute.Value);
    }

    private void AddSubElement(HElement subElement)
    {
      element.AppendChild(subElement);
    }
    public static implicit operator HtmlElement(HElement el)
    {
      return (el.element);
    }
    HtmlElement element;
  }
}